import os
import sys
import argparse

from . import utils
from . import linting


def get_cli_parser():
    parser = argparse.ArgumentParser(description='Copyright linter')

    parser.add_argument(
        '--workdir',
        type=str,
        default=os.getcwd(),
        help='Work directory',
    )

    return parser


def main():
    try:
        cli = get_cli_parser()
        workdir = cli.parse_args().workdir
        config = utils.load_config(workdir)
        linting.lint_project(workdir, config)
    except Exception as e:
        sys.exit(e)


if __name__ == '__main__':
    main()



import re

from . import utils


COPYRIGHT = '\n'.join([
    r"Copyright \(c\) 20\d{2} Celadon Development LLC, All rights reserved.",
    r"Author \w+ \w+ <\w+\.\w+@celadon.ae>",
])


def _comment_copyright(copyright, comment_symbol):
    copyright_lines = copyright.split('\n')
    commented_copyrights_lines = [
        ' '.join([comment_symbol, line])
        for line in copyright_lines
    ]
    return '\n'.join(commented_copyrights_lines)


COPYRIGHT_BY_FILE_EXTENSION = {
    'js': _comment_copyright(COPYRIGHT, '//'),
    'py': _comment_copyright(COPYRIGHT, '#'),
}


def get_file_copyright(file_path):
    file_extension = file_path.split('.')[-1]
    return COPYRIGHT_BY_FILE_EXTENSION[file_extension]


class LintingError(ValueError):
    def __init__(self, errors):
        if isinstance(errors, (list, tuple)):
            error_files = '\n'.join(errors)
        else:
            error_files = errors

        error_message = 'Failed linting such files:\n {}'.format(error_files)
        super().__init__(error_message)


def lint_project(workdir, config):
    failed_liniting_files = []
    for file_path in utils.walk_files_recursively(workdir):
        if not utils.is_ignore_file(file_path, config.ignores):
            try:
                lint_file(file_path)
            except LintingError:
                failed_liniting_files.append(file_path)
            except Exception:
                pass

    if failed_liniting_files:
        raise LintingError(failed_liniting_files)


def lint_file(file_path):
    with open(file_path, 'r') as file:
        source_code = file.read()
        copyright_re = re.compile(get_file_copyright(file_path), re.MULTILINE)
        if copyright_re.search(source_code) is None:
            raise LintingError(file_path)



import os
import fnmatch
import configparser
import collections


Config = collections.namedtuple('Config', ['ignores', ])

CONFIG_NAME = '.crlintrc'


def load_config(workdir):
    if not os.path.exists(os.path.join(workdir, CONFIG_NAME)):
        raise FileNotFoundError(
            'Cannot found {config} config'.format(config=CONFIG_NAME)
        )

    config = configparser.ConfigParser()
    config.read(os.path.join(workdir, CONFIG_NAME))

    config_ignore_patterns = [
        pattern.strip()
        for pattern in config.get('ignore', 'patterns').split(',')
    ]

    return Config(ignores=config_ignore_patterns)


def walk_files_recursively(root_path):
    for root, _, files in os.walk(root_path):
        for file in files:
            yield os.path.join(root, file)


def is_ignore_file(file_path, ignore_patterns):
    return any(
        fnmatch.fnmatch(file_path, ignore_pattern)
        for ignore_pattern in ignore_patterns
    )



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='crlint',
    version='0.0.1',
    author='Celadon Team',
    author_email='opensource@celadon.ae',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://bitbucket.org/celadonteam/crtool',
    packages=[
        'crlint',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={
        'console_scripts': [
            'crlint=crlint.__main__:main',
        ],
    },
)
